package com.abc.db;
import java.util.*;

public class MainThread extends Thread 
{
	private Hashtable	m_mapRuns;
	private Object		m_lock;
	
	protected MainThread()
	{
		m_lock = new Object();
		m_mapRuns = new Hashtable();
	}
	
	public void run()
	{
		long now;
		long lTime;
		Long oLong;
		ThreadFunc threadFunc = null;
		while(true)
		{
			now = System.currentTimeMillis();
			synchronized(this)
			{
				if( m_mapRuns.isEmpty() )
				{
					//we will  wait till insert
					synchronized( m_lock )
					{
						try { m_lock.wait(); } catch (InterruptedException e ) {}
						continue;
					}
				}
				Enumeration e = m_mapRuns.keys();
				oLong = (Long)e.nextElement();
				lTime = oLong.longValue();
				if( lTime <= now )
				{
					//we need to run this method.
					threadFunc = (ThreadFunc)m_mapRuns.get( oLong );
					threadFunc.m_bRunning =true;
				}
			}
			if( lTime > now )
			{
				synchronized( m_lock )
				{
					try { m_lock.wait( lTime - now ); } catch (InterruptedException e ) {}
					// we could exit wait because new element has been inserted
					// so we need to go check again.
					continue;
				}
			}
			// now we need to run our function.
			threadFunc.m_runable.run();
			synchronized(this)
			{
				m_mapRuns.remove( oLong );
				threadFunc.m_bRunning = false;
				threadFunc.m_nextTimeToRun = new Long( System.currentTimeMillis() + threadFunc.m_lWaitTime );
				m_mapRuns.put( threadFunc.m_nextTimeToRun, threadFunc );
				this.notify();
			}
		}
	}
	
	public void insert( Runnable run, long lWaitTime, boolean bRunImmediately )
	{
		synchronized(this)
		{
			ThreadFunc threadFunc = new ThreadFunc( run, lWaitTime, bRunImmediately );
			m_mapRuns.put( threadFunc.m_nextTimeToRun, threadFunc );
			synchronized( m_lock )
			{
				m_lock.notify();
			}
		}
	}
	
	public synchronized void remove( Runnable run )
	{
		for (Enumeration e = m_mapRuns.elements() ; e.hasMoreElements() ;) 
		{
			ThreadFunc threadFunc = (ThreadFunc)e.nextElement();
			if( threadFunc.m_runable == run )
			{
				if( threadFunc.m_bRunning == true )
				{
					try
					{
						this.wait();
					}
					catch( InterruptedException exc )
					{
						System.err.println(" Interuppted exception in MainThread - " + exc.getMessage() );
					}
				}
				m_mapRuns.remove( threadFunc.m_nextTimeToRun );
			}
		 }
	}
}
