package com.abc.db;
import java.sql.*;
import java.lang.Exception.*;
import java.util.*;

public class PoolOfConnections implements Runnable
{
	private int m_iMax = 10;
	private int m_iMin = 5;
	private final long m_lWaitTime = 60000; //time in miliseconds, 60000 - 1 minute
	private String m_sSystemUrl;
	private String m_sUser;
	private String m_sPassword;
	private Vector m_vecConnections = new Vector();
	
	public int getMin() {
		return m_iMin;
	}
	public void setMin( int iMin ) {
		m_iMin = iMin;
	}
	public int getMax() {
		return m_iMax;
	}
	public void setMax( int iMax ) {
		m_iMax = iMax;
	}
	public String getSystem() {
		return m_sSystemUrl;
	}
	public String getUser() {
		return m_sUser;
	}
	public int getConectionAmount() {
		return m_vecConnections.size();
	}
	public int getNotReleasedConnectionAmount() 
	{
		int busy = 0;
		PoolableConnection con;		
		synchronized(this)
		{
			for (Enumeration e = m_vecConnections.elements() ; e.hasMoreElements() ;) 
			{
				con = (PoolableConnection)e.nextElement();
				if( con.isBusy() )
					busy++;
			}		
		}
		return busy;
	}
	
	protected PoolOfConnections( String sSystemUrl, String sUser, String sPassword )
	{
		ThreadFuncManager.insert( this, m_lWaitTime, true );
		m_sSystemUrl = sSystemUrl;
		m_sUser =sUser;
		m_sPassword = sPassword;
	}
	
	protected final int connectionsAmount()
	{
		return m_vecConnections.size();
	}
	
	protected final Connection getConnection() 
		throws SQLException
	{
		PoolableConnection con;
		//System.err.println("Pool size - " + m_vecConnections.size());
		while( true )
		{
			synchronized(this)
			{
				for (Enumeration e = m_vecConnections.elements() ; e.hasMoreElements() ;) 
				{
					con = (PoolableConnection)e.nextElement();
					if( !con.isBusy() )
					{	
						con.setBusy(true);//we have found connection
						return con;  //we have found connection
					}
				}		
				if( m_vecConnections.size() >= m_iMax )
				{
					try{this.wait();} catch(Exception e ) {}
					continue;
				}
				else
				{
					con = new PoolableConnection();
					con.setBusy(true);
					m_vecConnections.addElement( con );
				}
			}
			con.initConnection(this, m_sSystemUrl, m_sUser, m_sPassword);
			return con;
		}
	}
	
	protected final void free( PoolableConnection con )
		throws SQLException
	{
		synchronized(this)
		{
			int iSize = m_vecConnections.size();
			int i = m_vecConnections.indexOf( con );
			con.setBusy( false );		
			if( m_vecConnections.size() >= m_iMax )
				this.notify();
		}
	}
	
	public final void run()
	{
		//here we will add new connections to pool if there is less than m_iMin.
		try
		{
			while( m_vecConnections.size() < m_iMin )
			{
				PoolableConnection con = new PoolableConnection();
				con.initConnection( this, m_sSystemUrl, m_sUser, m_sPassword );
				m_vecConnections.addElement( con );
			}		
			long lNow = System.currentTimeMillis();
			Vector m_toClose = null;
			synchronized(this)
			{
				int iSize = m_vecConnections.size();
				if( iSize <= m_iMin )
					return;
				for(int i = iSize -1; i >= 0; i--)
				{
					PoolableConnection con = (PoolableConnection)m_vecConnections.elementAt(i);
					if( !con.isBusy() )
					{
						if( lNow - con.m_lAccessTime > m_lWaitTime )
						{
							//System.err.println(" Go down - " +  m_vecConnections.size());
							//need to close this connection.
							if( m_toClose == null )
								m_toClose = new Vector();
							m_toClose.addElement(con);
							m_vecConnections.removeElementAt(i);
							i--; //we need to reduce i since we have that connection removed.
							iSize--;
							this.notify();
							if( iSize <= m_iMin )
								break;
						}
					}
				}
			}
			if( m_toClose != null )
			{
				for(int i = 0; i < m_toClose.size(); i++)
				{
					PoolableConnection con = (PoolableConnection)m_toClose.elementAt(i);
					con.closeConnection();
				}
			}
		}
		catch( Exception e )
		{
			System.err.println( "Exception in database pool - " + e.getMessage() );
		}
	}
}
