package com.abc.db;
import java.sql.*;
import java.lang.Exception.*;

public class PoolableConnection implements Connection
{
	private Connection			m_conConnection;
	private boolean				m_bBusy;
	private	PoolOfConnections	m_poolOfConnections;
	protected long				m_lAccessTime;
	
	protected PoolableConnection( )
	{
	}
	
	protected void initConnection(	  PoolOfConnections pool, 
								  String sSystem, 
								  String sUser, 
								  String sPassword )
		throws SQLException
	{
		m_poolOfConnections = pool;
		m_lAccessTime = System.currentTimeMillis();
		m_bBusy = false;
		if( sUser == null )
			m_conConnection = DriverManager.getConnection(sSystem);
		else
			m_conConnection = DriverManager.getConnection(sSystem, sUser, sPassword);
	}

	protected final void setBusy( boolean bBusy )
	{
		if( bBusy == false )
			m_lAccessTime = System.currentTimeMillis();
		m_bBusy = bBusy;
	}
	protected final boolean isBusy()
	{
		return m_bBusy;
	}
	protected final void closeConnection()
		throws SQLException
	{
		m_poolOfConnections = null;
		m_conConnection.close();
		m_conConnection = null;
	}
	
	//Connection Interface
	public final DatabaseMetaData getMetaData()
		throws SQLException 
	{
		return m_conConnection.getMetaData();
	}
	public final void rollback() 
		throws SQLException 
	{
		m_conConnection.rollback();
	}
	public final void setAutoCommit( boolean bAutoCommit ) 
		throws SQLException 
	{
		m_conConnection.setAutoCommit( bAutoCommit );
	}
	
	public final void close() 
		throws SQLException 
	{
		//we can not set m_bBusy to false becasue Pool will decide close this connection 
		//or make it available to others.
		m_poolOfConnections.free(this);
	}
	public final void commit() 
		throws SQLException 
	{
		m_conConnection.commit();
	}
	public final void setReadOnly( boolean bReadOnly ) 
		throws SQLException 
	{
		m_conConnection.setReadOnly( bReadOnly );
	}
	public final void setCatalog( String sCatalog ) 
		throws SQLException 
	{
		m_conConnection.setCatalog( sCatalog );
	}
	public final void setTransactionIsolation( int iTransactionIsolation ) 
		throws SQLException 
	{
		m_conConnection.setTransactionIsolation( iTransactionIsolation );
	}
	public final void clearWarnings()	
		throws SQLException 
	{
		m_conConnection.clearWarnings();
	}
	public final int getTransactionIsolation() 
		throws SQLException 
	{
		return m_conConnection.getTransactionIsolation();
	}
	public final Statement createStatement() 
		throws SQLException 
	{
		return m_conConnection.createStatement();
	}
	public final boolean isReadOnly() 
		throws SQLException 
	{
		return m_conConnection.isReadOnly();
	}
	public final SQLWarning getWarnings() 
		throws SQLException 
	{
		return m_conConnection.getWarnings();
	}
	public final String getCatalog() 
		throws SQLException 
	{
		return m_conConnection.getCatalog();
	}
	public final String nativeSQL( String sNativeSQL) 
		throws SQLException 
	{
		return m_conConnection.nativeSQL( sNativeSQL );
	}
	public final PreparedStatement prepareStatement( String sPreparedStatement ) 
		throws SQLException 
	{
		return m_conConnection.prepareStatement( sPreparedStatement );
	}
	public final CallableStatement prepareCall( String sSql ) 
		throws SQLException 
	{
		return m_conConnection.prepareCall( sSql );
	}
	public final boolean isClosed() 
		throws SQLException 
	{
		return m_conConnection.isClosed();
	}
	public final boolean getAutoCommit() 
		throws SQLException 
	{
		return m_conConnection.getAutoCommit();
	}	
}
